package org.ow2.orchestra.designer.bpmn.views {
  import com.adobe.cairngorm.CairngormError;
  import com.adobe.cairngorm.CairngormMessageCodes;

  import flash.display.DisplayObject;
  import flash.geom.Point;
  import flash.geom.Rectangle;

  import mx.binding.utils.ChangeWatcher;
  import mx.collections.ArrayCollection;
  import mx.containers.VBox;

  import org.ow2.orchestra.designer.bpmn.model.AbstractElement;

  /**
   * Common abstract class for process elements views
   *
   */
  public class AbstractElementView extends VBox {
    protected var watchers:ArrayCollection = new ArrayCollection();

    function AbstractElementView() {
    }

    /**
     * Returns the model object for this view.
     *
     * Note: This method is designed to be abstract,
     * sub-classes MUST override it.
     * As Abstract method/classes support appear with Flex 4,
     * This method throw an IllegalOperationError if called
     * directly.
     */
    public function get model():AbstractElement {
      //This method is designed to be abstract
      throw new CairngormError(CairngormMessageCodes.ABSTRACT_METHOD_CALLED);
    }

    /**
     * Returns a new view for the same model object.
     *
     * Note: This method is designed to be abstract,
     * sub-classes MUST override it.
     * As Abstract method/classes support appear with Flex 4,
     * This method throw an IllegalOperationError if called
     * directly.
     */
    public function copy():AbstractElementView {
      //This method is designed to be abstract
      throw new CairngormError(CairngormMessageCodes.ABSTRACT_METHOD_CALLED);
    }

    /**
     * Returns the image which delimits the border for this view.
     *
     * Note: This method is designed to be abstract,
     * sub-classes MUST override it.
     * As Abstract method/classes support appear with Flex 4,
     * This method throw an IllegalOperationError if called
     * directly.
     */
    public function get borderImage():DisplayObject {
      //This method is designed to be abstract
      throw new CairngormError(CairngormMessageCodes.ABSTRACT_METHOD_CALLED);
    }

    /**
     * Returns the image which defines the subtype for this view.
     *
     * Note: This method is designed to be abstract,
     * sub-classes MUST override it.
     * As Abstract method/classes support appear with Flex 4,
     * This method throw an IllegalOperationError if called
     * directly.
     */
    public function get subtypeImage():DisplayObject {
        //This method is designed to be abstract
        throw new CairngormError(CairngormMessageCodes.ABSTRACT_METHOD_CALLED);
    }

    public function dispose():void {
      for each (var changeWatcher:ChangeWatcher in watchers) {
        if (changeWatcher.isWatching()) {
          changeWatcher.unwatch();
        }
      }
    }

    /**
     * This method allow to know if a rectangle is valid to select this object.
     *
     * This implementation retrieves the rectangle that defines the boundary of this display object. If the selection
     * rectangle contains this rectangle, then the display object is selected by the selection rectangle.
     *
     * This method can be overridden by subclasses.
     *
     * {@see DisplayObject#getRect}
     *
     * @param selection A {@link Rectangle} representing the selection
     * @param targetCoordinateSpace The {@link DisplayObject} from which <code>selection</code> coordinates are relative
     * @return Returns true if the selection rectangle is valid to select the object false otherwise
     */
    public function includeInto(selection:Rectangle, targetCoordinateSpace:DisplayObject):Boolean {
      // get the child object display zone (with coordinates relative to this view)
      var bounds:Rectangle = this.getRect(targetCoordinateSpace);

      if (selection.containsRect(bounds)) {
        return true;
      }
      return false;
    }

    /**
     * This method allow to know if a rectangle is valid to select this object.
     *
     * This implementation retrieves the rectangle that defines the boundary of this display object. If this
     * rectangle contains the selection rectangle, then the display object is selected by the selection rectangle.
     *
     * This method can be overridden by subclasses.
     *
     * {@see DisplayObject#getRect}
     *
     * @param selection A {@link Rectangle} representing the selection
     * @param targetCoordinateSpace The {@link DisplayObject} from which <code>selection</code> coordinates are relative
     * @return Returns true if the selection rectangle is valid to select the object false otherwise
     */
    public function containsRectangle(selection:Rectangle, targetCoordinateSpace:DisplayObject):Boolean {
      // get the child object display zone (with coordinates relative to this view)
      var objectRect:Rectangle = this.getRect(targetCoordinateSpace);

      if (objectRect.containsRect(selection)) {
        return true;
      }
      return false;
    }

    public function containsPoint(point:Point, targetCoordinateSpace:DisplayObject):Boolean {
      var selectionRectangle:Rectangle = new Rectangle(point.x, point.y);
      return this.containsRectangle(selectionRectangle, targetCoordinateSpace);
    }
  }
}
