/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.ow2.orchestra.designer.bpmn.views {
import flash.display.DisplayObject;
import flash.events.Event;
import flash.geom.Rectangle;

import mx.binding.utils.BindingUtils;
import mx.binding.utils.ChangeWatcher;
import mx.containers.Canvas;
import mx.controls.Image;

import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.bpmn.model.GatewayModel;
import org.ow2.orchestra.designer.bpmn.model.GatewayType;

public class GatewayView extends AbstractElementView implements IFlowNodeView  {

    /**
     * The source for "gateway" image.
     */
    [Embed(source="/../assets/gateway.svg")]
    public static const gatewayImg:Class;
    /**
     * The source for "parallel gateway" image.
     */
    [Embed(source="/../assets/parallelGateway.svg")]
    public static const parallelGatewayImg:Class;
    /**
     * The source for "exclusive gateway" image.
     */
    [Embed(source="/../assets/exclusiveGateway.svg")]
    public static const exclusiveGatewayImg:Class;
    /**
     * The source for "exclusive gateway" image.
     */
    [Embed(source="/../assets/eventBasedGateway.svg")]
    public static const exclusiveEventGatewayImg:Class;

    [Bindable]
    private var _gatewayModel:GatewayModel;

    private var imageGateway:Image = new Image();
    private var imageGatewayBackground:Image = new Image();

    function GatewayView(gatewayModel:GatewayModel) {
      _gatewayModel = gatewayModel;
      this.setStyle("horizontalAlign", "center");
      imageGatewayBackground.source = gatewayImg;
      imageGatewayBackground.scaleContent = false;
      imageGatewayBackground.automationName = "GatewayBackgroundImage" + _gatewayModel.id;
      imageGateway.automationName = "GatewayImage" + _gatewayModel.id;
      imageGateway.scaleContent = false;

      watchers.addItem(BindingUtils.bindProperty(this, "x", _gatewayModel, "x"));
      watchers.addItem(BindingUtils.bindProperty(this, "y", _gatewayModel, "y"));
      watchers.addItem(ChangeWatcher.watch(_gatewayModel, "gatewayType", updateImage));

      var imageCanvas:Canvas = new Canvas();
      imageCanvas.addChild(imageGatewayBackground);
      imageCanvas.addChild(imageGateway);
      this.addChild(imageCanvas);
      this.updateImage();
    }

    protected function updateImage(event:Event = null):void {
      if (_gatewayModel.gatewayType == GatewayType.PARALLEL) {
        imageGateway.source = parallelGatewayImg;
      } else if (_gatewayModel.gatewayType == GatewayType.EXCLUSIVE) {
        imageGateway.source = exclusiveGatewayImg;
      } else if (_gatewayModel.gatewayType == GatewayType.EXCLUSIVE_EVENT) {
        imageGateway.source = exclusiveEventGatewayImg;
      }
    }

    override public function get model():AbstractElement {
      return _gatewayModel;
    }

    override public function copy():AbstractElementView {
      return new GatewayView(_gatewayModel);
    }

    override public function get borderImage():DisplayObject {
      return imageGatewayBackground;
    }

    override public function get subtypeImage():DisplayObject {
        return imageGateway;
    }

    public function getCollisionRectangle(targetCoordinateSpace:DisplayObject):Rectangle {
      return this.borderImage.getRect(targetCoordinateSpace);
    }
  }
}
