/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.bpmn.views {
  import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
  import org.ow2.orchestra.designer.bpmn.model.EventModel;
  import org.ow2.orchestra.designer.bpmn.model.GatewayModel;
  import org.ow2.orchestra.designer.bpmn.model.PoolModel;
  import org.ow2.orchestra.designer.bpmn.model.SequenceFlowModel;
  import org.ow2.orchestra.designer.bpmn.model.TaskModel;

  /**
   * Factory of bpmn views.
   *
   * @author Loic Albertin
   *
   */
  public class ViewFactory {
    public function ViewFactory() {
    }

    /**
     * Instantiates an AbstractElementView according to the given model type.
     * For example if the given model is an instance of @see(org.ow2.orchestra.designer.bpmn.model.GatewayModel)
     * this method will returns an instance of @see(org.ow2.orchestra.designer.bpmn.views.GatewayView).
     *
     * <b>Warning:</b> This method also associate (using view constructor) the view with the <b>given</b> model.
     *
     * @param model the model for which a corresponding view should be instantiated
     * @return returns an element view instance according to the given model
     *
     */
    public static function getViewByModel(model:AbstractElement, processView:ProcessView):AbstractElementView {
      var view:AbstractElementView = null;
      if (model is GatewayModel) {
        view = new GatewayView(model as GatewayModel);
      } else if (model is EventModel) {
        view = new EventView(model as EventModel);
      } else if (model is TaskModel) {
        view = new TaskView(model as TaskModel, processView);
      } else if (model is SequenceFlowModel) {
        if (processView != null) {
          view = new SequenceFlowView(model as SequenceFlowModel, processView);
        }
      } else if (model is PoolModel) {
        view = new PoolView(model as PoolModel);
      }
      // Add automationName for UI testing
      if (view != null) {
        view.automationName = "view" + model.id;
      }
      return view;
    }

  }
}
